"""
Json client handler
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015-2019 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long


import itertools
import six

from vmware.vapi.core import ApiProvider
from vmware.vapi.protocol.client.http_lib import HTTPMethod, HTTPRequest
from vmware.vapi.protocol.client.msg.generic_connector import GenericConnector
from vmware.vapi.lib.constants import (HTTP_USER_AGENT_HEADER, JSON_CONTENT_TYPE,    # pylint: disable=line-too-long
                                       HTTP_CONTENT_TYPE_HEADER)
from vmware.vapi.lib.load import dynamic_import_list
from vmware.vapi.lib.log import get_client_wire_logger, get_vapi_logger
from vmware.vapi.protocol.client.msg.user_agent_util import get_user_agent

from vmware.vapi.data.serializers.jsonrpc import (
    JsonRpcDictToVapi,
    vapi_jsonrpc_request_factory, deserialize_response,
    vapi_jsonrpc_error_transport_error,
    VAPI_INVOKE
)

logger = get_vapi_logger(__name__)
request_logger = get_client_wire_logger()


class JsonClientProvider(ApiProvider):
    """ Json rpc client provider """

    def __init__(self, http_provider, post_processors):
        """
        Json rpc client provider init

        :type  http_provider:
            :class:`vmware.vapi.protocol.client.rpc.provider.HTTPProvider`
        :param http_provider: rpc provider object
        :type  post_processors: :class:`list` of :class:`str`
        :param post_processors: List of post processor class names
        """

        ApiProvider.__init__(self)
        self.http_provider = http_provider
        self.counter = itertools.count()
        self.to_vapi = JsonRpcDictToVapi

        # Load all the post processors
        self.post_processors = [
            constructor()
            for constructor in dynamic_import_list(post_processors)]

    def invoke(self, service_id, operation_id, input_value, ctx):
        """
        Invokes the specified method using the input value and the
        the execution context provided

        :type  service_id: :class:`str`
        :param service_id: Service identifier
        :type  operation_id: :class:`str`
        :param operation_id: Operation identifier
        :type  input_value: :class:`vmware.vapi.data.value.DataValue`
        :param input_value: method input parameters
        :type  ctx: :class:`vmware.vapi.core.ExecutionContext`
        :param ctx: execution context object
        :rtype: :class:`vmware.vapi.core.MethodResult`
        :return: method result object
        """

        params = {
            'serviceId': service_id,
            'operationId': operation_id,
            'input': input_value,
            'ctx': ctx,
        }
        response = self._do_request(VAPI_INVOKE, ctx, params)
        result = self.to_vapi.method_result(response.result)
        return result


    #
    ## vapi methods end

    def _append_jsonrps_1_1_headers(self, headers, svc_name, op_name, ctx):
        """
        Append JSON-RPC v1.1 headers

        :type  headers: :class:`dict`
        :param method: headers to which JSON-RPC 1.1 headers will be appended
        :type  svc_name: : string
        :param svc_name: name of vAPI service which is invoked
        :type  op_name: : string
        :param op_name: name of vAPI operation which is invoked
        :type  ctx: :class:`vmware.vapi.core.ExecutionContext`
        :param ctx: execution context object

        check: https://confluence.eng.vmware.com/display/VAPI/JSON-RPC+v1.1
        """
        headers['vapi-service'] = svc_name
        headers['vapi-operation'] = op_name

        def convert_name(key):
            key = key.lower()
            return 'vapi-ctx-%s' % key

        for key, value in ctx.application_context.items():
            headers[convert_name(key)] = value


    def _do_request(self, method, ctx, params=None):
        """
        Perform json rpc request

        :type  method: :class:`str`
        :param method: json rpc method name
        :type  ctx: :class:`vmware.vapi.core.ExecutionContext`
        :param ctx: execution context object
        :type  params: :class:`dict` or None
        :param params: json rpc method params
        :rtype: :class:`vmware.vapi.data.serializer.jsonrpc.JsonRpcResponse`
        :return: json rpc response
        """

        logger.debug('_do_request: request %s', method)

        if not self.http_provider.connect():
            logger.error('Connection refused')
            raise vapi_jsonrpc_error_transport_error()
        request_headers = {HTTP_CONTENT_TYPE_HEADER: JSON_CONTENT_TYPE,
                           HTTP_USER_AGENT_HEADER: get_user_agent()}
        if params is not None:
            self._append_jsonrps_1_1_headers(request_headers, params['serviceId'],
                                             params['operationId'], ctx)

        id_ = six.advance_iterator(self.counter)    # atomic increment
        id_ = str(id_)    # TODO: Bypass java barf temporary
        request = vapi_jsonrpc_request_factory(method=method,
                                               params=params,
                                               id=id_)
        request_body = request.serialize()
        if six.text_type:
            request_body = request_body.encode('utf-8')
        for processor in self.post_processors:
            request_body = processor.process(request_body)

        request_logger.debug('_do_request: request %s', request_body)
        # do_request returns http_response
        http_response = self.http_provider.do_request(
            HTTPRequest(method=HTTPMethod.POST, url_path=None,
                        headers=request_headers, body=request_body))
        if http_response.data is not None:
            # Currently only RequestsProvider returns the requests Response
            # object as data back. We need to raise exception if error is
            # returned to keep existing behavior.
            http_response.data.raise_for_status()
        request_logger.debug('_do_request: response %s', http_response.body)

        if ctx.runtime_data is not None:
            response_extractor = ctx.runtime_data.get('response_extractor')
            if response_extractor is not None:
                response_extractor.set_http_status(http_response.status)
                response_extractor.set_http_headers(http_response.headers)
                response_extractor.set_http_body(http_response.body)
                response_extractor.set_http_method(HTTPMethod.POST)
                response_extractor.set_http_url(self.http_provider._base_url)    # pylint: disable=protected-access

        response = deserialize_response(http_response.body)
        request.validate_response(response)
        if response.error is not None:
            logger.error('_do_request: method %s response with error %s',
                         method, response.error)
            raise response.error  # pylint: disable=E0702
        return response


def get_protocol_connector(
        http_provider, post_processors=None, provider_filter_chain=None):
    """
    Get protocol connector

    :type  http_provider:
        :class:`vmware.vapi.protocol.client.rpc.provider.HTTPProvider`
    :param http_provider: rpc provider object
    :type  post_processors: :class:`list` of :class:`str`
    :param post_processors: List of post processor class names
    :type  provider_filter_chain: :class:`list` of
        :class:`vmware.vapi.provider.filter.ApiProviderFilter`
    :param provider_filter_chain: List of API filters in order they are to be
        chained
    :rtype: :class:`vmware.vapi.protocol.client.connector.Connector`
    :return: json rpc connector object
    """
    if post_processors is None:
        post_processors = []
    api_provider = JsonClientProvider(http_provider, post_processors)
    connector = GenericConnector(
        http_provider, api_provider, provider_filter_chain)
    return connector




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.
#---------------------------------------------------------------------------

"""
The
``com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs_client``
module provides classes to manage the vCenter Server Inventory Cluster DRS
settings.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata

class ScaleSharesBehavior(Enum):
    """
    

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    DISABLED = None
    """
    Do not scale shares

    """
    SCALE_CPU_AND_MEMORY_SHARES = None
    """
    Scale both CPU and memory shares

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`ScaleSharesBehavior` instance.
        """
        Enum.__init__(string)

ScaleSharesBehavior._set_values({
    'DISABLED': ScaleSharesBehavior('DISABLED'),
    'SCALE_CPU_AND_MEMORY_SHARES': ScaleSharesBehavior('SCALE_CPU_AND_MEMORY_SHARES'),
})
ScaleSharesBehavior._set_binding_type(type.EnumType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.scale_shares_behavior',
    ScaleSharesBehavior))



class DrsBehaviorInfo(Enum):
    """
    The ``DrsBehaviorInfo`` class defines the automation levels that can be set
    on a DRS cluster.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    MANUAL = None
    """
    Specifies that VirtualCenter should generate recommendations for virtual
    machine migration and for placement with a host, but should not implement
    the recommendations automatically.

    """
    PARTIALLY_AUTOMATED = None
    """
    Specifies that VirtualCenter should generate recommendations for virtual
    machine migration and for placement with a host, but should automatically
    implement only the placement at power on.

    """
    FULLY_AUTOMATED = None
    """
    Specifies that VirtualCenter should automate both the migration of virtual
    machines and their placement with a host at power on.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`DrsBehaviorInfo` instance.
        """
        Enum.__init__(string)

DrsBehaviorInfo._set_values({
    'MANUAL': DrsBehaviorInfo('MANUAL'),
    'PARTIALLY_AUTOMATED': DrsBehaviorInfo('PARTIALLY_AUTOMATED'),
    'FULLY_AUTOMATED': DrsBehaviorInfo('FULLY_AUTOMATED'),
})
DrsBehaviorInfo._set_binding_type(type.EnumType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.drs_behavior_info',
    DrsBehaviorInfo))



class DpmBehaviorInfo(Enum):
    """
    The ``DpmBehaviorInfo`` class defines the automation level for DPM service
    on a cluster.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    MANUAL = None
    """
    Specifies that VirtualCenter should generate recommendations for host power
    operations, but should not execute the recommendations automatically.

    """
    AUTOMATED = None
    """
    Specifies that VirtualCenter should generate recommendations for host power
    operations, and should execute the recommendations automatically.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`DpmBehaviorInfo` instance.
        """
        Enum.__init__(string)

DpmBehaviorInfo._set_values({
    'MANUAL': DpmBehaviorInfo('MANUAL'),
    'AUTOMATED': DpmBehaviorInfo('AUTOMATED'),
})
DpmBehaviorInfo._set_binding_type(type.EnumType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.dpm_behavior_info',
    DpmBehaviorInfo))



class Status(Enum):
    """
    The Status enumeration defines a general "health" value for a managed
    entity.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    GRAY = None
    """
    The status is unknown.

    """
    GREEN = None
    """
    The entity is OK.

    """
    YELLOW = None
    """
    The entity might have a problem.

    """
    RED = None
    """
    The entity definitely has a problem.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`Status` instance.
        """
        Enum.__init__(string)

Status._set_values({
    'GRAY': Status('GRAY'),
    'GREEN': Status('GREEN'),
    'YELLOW': Status('YELLOW'),
    'RED': Status('RED'),
})
Status._set_binding_type(type.EnumType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.status',
    Status))




class AdditionalOptions(VapiStruct):
    """
    The ``AdditionalOptions`` class contains attributes describing the HA
    specific configurations of a cluster. It contains cluster-wide
    configurations for vSphereHA.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 advanced_options=None,
                 scalable_shares=None,
                ):
        """
        :type  advanced_options: :class:`list` of :class:`com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client.OptionValue` or ``None``
        :param advanced_options: Drs configuration additional options
        :type  scalable_shares: :class:`ScaleSharesBehavior` or ``None``
        :param scalable_shares: Enable scalable shares for the resource pools on this cluster.
        """
        self.advanced_options = advanced_options
        self.scalable_shares = scalable_shares
        VapiStruct.__init__(self)


AdditionalOptions._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.additional_options', {
        'advanced_options': type.OptionalType(type.ListType(type.ReferenceType('com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client', 'OptionValue'))),
        'scalable_shares': type.OptionalType(type.ReferenceType(__name__, 'ScaleSharesBehavior')),
    },
    AdditionalOptions,
    False,
    None))



class Automation(VapiStruct):
    """
    The ``Automation`` class contains attributes describing the HA specific
    configurations of a cluster. It contains cluster-wide configurations for
    vSphereHA.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 automation_level=None,
                 migration_threshold=None,
                 proactive_drs_enabled=None,
                 virtual_machine_automation=None,
                ):
        """
        :type  automation_level: :class:`DrsBehaviorInfo` or ``None``
        :param automation_level: Specifies the cluster-wide default DRS behavior for virtual
            machines. You can override the default behavior for a virtual
            machine.
            If None or empty, the value is skipped.
        :type  migration_threshold: :class:`long` or ``None``
        :param migration_threshold: Threshold for generated recommendations. DRS generates only those
            recommendations that are above the specified vmotionRate. Ratings
            vary from 1 to 5.
            If None or empty, the value is skipped.
        :type  proactive_drs_enabled: :class:`bool` or ``None``
        :param proactive_drs_enabled: Flag indicating whether or not the ProactiveDRS is enabled.
            If None or empty, the value is skipped.
        :type  virtual_machine_automation: :class:`bool` or ``None``
        :param virtual_machine_automation: Flag that dictates whether DRS Behavior overrides for individual
            VMs.
        """
        self.automation_level = automation_level
        self.migration_threshold = migration_threshold
        self.proactive_drs_enabled = proactive_drs_enabled
        self.virtual_machine_automation = virtual_machine_automation
        VapiStruct.__init__(self)


Automation._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.automation', {
        'automation_level': type.OptionalType(type.ReferenceType(__name__, 'DrsBehaviorInfo')),
        'migration_threshold': type.OptionalType(type.IntegerType()),
        'proactive_drs_enabled': type.OptionalType(type.BooleanType()),
        'virtual_machine_automation': type.OptionalType(type.BooleanType()),
    },
    Automation,
    False,
    None))



class DrsConfig(VapiStruct):
    """
    The ``DrsConfig`` class contains attributes describing the HA specific
    configurations of a cluster. It contains cluster-wide configurations for
    DAS.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 enabled=None,
                 automation=None,
                 additional_options=None,
                 power_management=None,
                ):
        """
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag indicating whether or not DRS service is enabled.
            If None or empty, the value is skipped.
        :type  automation: :class:`Automation` or ``None``
        :param automation: vSphere HA configuration for Automation Level, Migration Threshold
            Predictive DRS and VM Automation.
        :type  additional_options: :class:`AdditionalOptions` or ``None``
        :param additional_options: vSphere HA configuration for VM Distribution, CPU Over commit
            Scalable Shares.
        :type  power_management: :class:`PowerManagement` or ``None``
        :param power_management: vSphere HA configuration for DPM, AutomationLevel, DPM Threshold.
        """
        self.enabled = enabled
        self.automation = automation
        self.additional_options = additional_options
        self.power_management = power_management
        VapiStruct.__init__(self)


DrsConfig._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.drs_config', {
        'enabled': type.OptionalType(type.BooleanType()),
        'automation': type.OptionalType(type.ReferenceType(__name__, 'Automation')),
        'additional_options': type.OptionalType(type.ReferenceType(__name__, 'AdditionalOptions')),
        'power_management': type.OptionalType(type.ReferenceType(__name__, 'PowerManagement')),
    },
    DrsConfig,
    False,
    None))



class DrsVmOverrides(VapiStruct):
    """
    The ``DrsVmOverrides`` class contains the fields describing DRS behavior
    override for individual virtual machines in the cluster.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 name=None,
                 behavior=None,
                ):
        """
        :type  name: :class:`str` or ``None``
        :param name: Reference to the virtual machine.
        :type  behavior: :class:`DrsBehaviorInfo` or ``None``
        :param behavior: Specifies the particular DRS behavior for this virtual machine.
            If None or empty, the value is skipped.
        """
        self.name = name
        self.behavior = behavior
        VapiStruct.__init__(self)


DrsVmOverrides._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.drs_vm_overrides', {
        'name': type.OptionalType(type.StringType()),
        'behavior': type.OptionalType(type.ReferenceType(__name__, 'DrsBehaviorInfo')),
    },
    DrsVmOverrides,
    False,
    None))



class GroupDetails(VapiStruct):
    """


    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 name=None,
                 entities=None,
                 user_created=None,
                ):
        """
        :type  name: :class:`str` or ``None``
        :param name: Unique name of the group.
        :type  entities: :class:`list` of :class:`str` or ``None``
        :param entities: List of VMs or Hosts belonging to the group.
            If None or empty, the value is skipped.
        :type  user_created: :class:`bool` or ``None``
        :param user_created: Flag to indicate whether the group is created by the user or the
            system.
            If None or empty, the value is skipped.
        """
        self.name = name
        self.entities = entities
        self.user_created = user_created
        VapiStruct.__init__(self)


GroupDetails._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.group_details', {
        'name': type.OptionalType(type.StringType()),
        'entities': type.OptionalType(type.ListType(type.StringType())),
        'user_created': type.OptionalType(type.BooleanType()),
    },
    GroupDetails,
    False,
    None))



class Group(VapiStruct):
    """
    The ``Group`` class describes the properties of virtual machine and host
    groups.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 vm_groups=None,
                 host_groups=None,
                ):
        """
        :type  vm_groups: :class:`list` of :class:`GroupDetails` or ``None``
        :param vm_groups: List of VM Group details.
            If None or empty, the value is skipped.
        :type  host_groups: :class:`list` of :class:`GroupDetails` or ``None``
        :param host_groups: List of HOST Group details.
            If None or empty, the value is skipped.
        """
        self.vm_groups = vm_groups
        self.host_groups = host_groups
        VapiStruct.__init__(self)


Group._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.group', {
        'vm_groups': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'GroupDetails'))),
        'host_groups': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'GroupDetails'))),
    },
    Group,
    False,
    None))



class PowerManagement(VapiStruct):
    """
    The ``PowerManagement`` class contains the fields describing DPM specific
    configurations of a cluster.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 dpm_enabled=None,
                 automation_level=None,
                 dpm_threshold=None,
                ):
        """
        :type  dpm_enabled: :class:`bool` or ``None``
        :param dpm_enabled: Flag indicating whether or not the service is enabled. This service
            can not be enabled, unless DRS is enabled as well.
            If None or empty, the value is skipped.
        :type  automation_level: :class:`DpmBehaviorInfo` or ``None``
        :param automation_level: Specifies the default VMware DPM behavior for hosts. This default
            behavior can be overridden on a per host.
            If None or empty, the value is skipped.
        :type  dpm_threshold: :class:`com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client.OptionValue` or ``None``
        :param dpm_threshold: DPM Advanced options.
            If None or empty, the value is skipped.
        """
        self.dpm_enabled = dpm_enabled
        self.automation_level = automation_level
        self.dpm_threshold = dpm_threshold
        VapiStruct.__init__(self)


PowerManagement._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.power_management', {
        'dpm_enabled': type.OptionalType(type.BooleanType()),
        'automation_level': type.OptionalType(type.ReferenceType(__name__, 'DpmBehaviorInfo')),
        'dpm_threshold': type.OptionalType(type.ReferenceType('com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client', 'OptionValue')),
    },
    PowerManagement,
    False,
    None))



class ProactiveHAConfig(VapiStruct):
    """
    The ``ProactiveHAConfig`` class defines the Configuration of the vSphere
    InfraUpdateHA service on a cluster.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 enabled=None,
                 automation_level=None,
                 remediation=None,
                 providers=None,
                ):
        """
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag indicating whether or not the service is enabled.
            InfraUpdateHA will not be active, unless DRS is enabled as well.
            If None or empty, the value is skipped.
        :type  automation_level: :class:`ProactiveHAConfig.BehaviorType` or ``None``
        :param automation_level: Configured behavior. Values are of type
            InfraUpdateHaConfig.BehaviorType.
            If None or empty, the value is skipped.
        :type  remediation: :class:`ProactiveHAConfig.RemediationType` or ``None``
        :param remediation: Configured remediation for moderately degraded hosts. Values are of
            type InfraUpdateHaConfig.RemediationType.
            If None or empty, the value is skipped.
        :type  providers: :class:`list` of :class:`str` or ``None``
        :param providers: The list of health update providers configured for this cluster.
            Providers are identified by their id. If the provider list is
            empty, InfraUpdateHA will not be active.
            If None or empty, the value is skipped.
        """
        self.enabled = enabled
        self.automation_level = automation_level
        self.remediation = remediation
        self.providers = providers
        VapiStruct.__init__(self)


    class BehaviorType(Enum):
        """
        The ``ProactiveHAConfig.BehaviorType`` class defines the behavior for
        executing the proposed DRS recommendations.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        MANUAL = None
        """
        With this behavior configured, the proposed DRS recommendations require
        manual approval before they are executed.

        """
        AUTOMATED = None
        """
        With this behavior configured, the proposed DRS recommendations are
        executed immediately.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`BehaviorType` instance.
            """
            Enum.__init__(string)

    BehaviorType._set_values({
        'MANUAL': BehaviorType('MANUAL'),
        'AUTOMATED': BehaviorType('AUTOMATED'),
    })
    BehaviorType._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.proactive_HA_config.behavior_type',
        BehaviorType))

    class RemediationType(Enum):
        """
        The ``ProactiveHAConfig.RemediationType`` class defines the types of
        remediation behaviours that can be configured.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        QUARANTINE_MODE = None
        """
        With this behavior configured, a degraded host will be recommended to be
        placed in Quarantine Mode.

        """
        MAINTENANCE_MODE = None
        """
        With this behavior configured, a degraded host will be recommended to be
        placed in Maintenance Mode.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`RemediationType` instance.
            """
            Enum.__init__(string)

    RemediationType._set_values({
        'QUARANTINE_MODE': RemediationType('QUARANTINE_MODE'),
        'MAINTENANCE_MODE': RemediationType('MAINTENANCE_MODE'),
    })
    RemediationType._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.proactive_HA_config.remediation_type',
        RemediationType))

ProactiveHAConfig._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.proactive_HA_config', {
        'enabled': type.OptionalType(type.BooleanType()),
        'automation_level': type.OptionalType(type.ReferenceType(__name__, 'ProactiveHAConfig.BehaviorType')),
        'remediation': type.OptionalType(type.ReferenceType(__name__, 'ProactiveHAConfig.RemediationType')),
        'providers': type.OptionalType(type.ListType(type.StringType())),
    },
    ProactiveHAConfig,
    False,
    None))



class AffinityRule(VapiStruct):
    """
    The ``AffinityRule`` class defines a set of virtual machines that DRS will
    attempt to run on the same host.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 status=None,
                 enabled=None,
                 in_compliance=None,
                 name=None,
                 mandatory=None,
                 user_created=None,
                 vms=None,
                ):
        """
        :type  status: :class:`Status` or ``None``
        :param status: Flag to indicate whether or not the rule is currently satisfied.
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag to indicate whether or not the rule is enabled. Set this
            property when user configures the rule. The default value is false
            (disabled). If there is a rule conflict, the Server can override
            the setting to disable a rule.
            If None or empty, the value is skipped.
        :type  in_compliance: :class:`bool` or ``None``
        :param in_compliance: Flag to indicate if the rule is in compliance.
            If None or empty, the value is skipped.
        :type  name: :class:`str` or ``None``
        :param name: Name of the rule.
        :type  mandatory: :class:`bool` or ``None``
        :param mandatory: Flag to indicate whether compliance with this rule is mandatory or
            optional. The default value is false (optional).
            If None or empty, the value is skipped.
        :type  user_created: :class:`bool` or ``None``
        :param user_created: Flag to indicate whether the rule is created by the user or the
            system.
            If None or empty, the value is skipped.
        :type  vms: :class:`list` of :class:`str` or ``None``
        :param vms: List of virtual machines.
        """
        self.status = status
        self.enabled = enabled
        self.in_compliance = in_compliance
        self.name = name
        self.mandatory = mandatory
        self.user_created = user_created
        self.vms = vms
        VapiStruct.__init__(self)


AffinityRule._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.affinity_rule', {
        'status': type.OptionalType(type.ReferenceType(__name__, 'Status')),
        'enabled': type.OptionalType(type.BooleanType()),
        'in_compliance': type.OptionalType(type.BooleanType()),
        'name': type.OptionalType(type.StringType()),
        'mandatory': type.OptionalType(type.BooleanType()),
        'user_created': type.OptionalType(type.BooleanType()),
        'vms': type.OptionalType(type.ListType(type.StringType())),
    },
    AffinityRule,
    False,
    None))



class DependencyRule(VapiStruct):
    """
    The ``DependencyRule`` class defines VM-to-VM dependencies. 
    
    A VM-VM Dependency rule identifies the following groups. 
    
    * A virtual machine group - :attr:`DependencyRule.vm_group`
    * A "depends on" virtual machine group -
      :attr:`DependencyRule.depends_on_vm_group`.
    
    
    
    The VMs in :attr:`DependencyRule.vm_group` depends on the list of VMs
    specified in :attr:`DependencyRule.depends_on_vm_group`.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 status=None,
                 enabled=None,
                 in_compliance=None,
                 name=None,
                 mandatory=None,
                 user_created=None,
                 vm_group=None,
                 depends_on_vm_group=None,
                ):
        """
        :type  status: :class:`Status` or ``None``
        :param status: Flag to indicate whether or not the rule is currently satisfied.
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag to indicate whether or not the rule is enabled. Set this
            property when user configures the rule. The default value is false
            (disabled). If there is a rule conflict, the Server can override
            the setting to disable a rule.
            If None or empty, the value is skipped.
        :type  in_compliance: :class:`bool` or ``None``
        :param in_compliance: Flag to indicate if the rule is in compliance.
            If None or empty, the value is skipped.
        :type  name: :class:`str` or ``None``
        :param name: Name of the rule.
        :type  mandatory: :class:`bool` or ``None``
        :param mandatory: Flag to indicate whether compliance with this rule is mandatory or
            optional. The default value is false (optional).
            If None or empty, the value is skipped.
        :type  user_created: :class:`bool` or ``None``
        :param user_created: Flag to indicate whether the rule is created by the user or the
            system.
            If None or empty, the value is skipped.
        :type  vm_group: :class:`str` or ``None``
        :param vm_group: Virtual group name. The virtual group may contain one or more
            virtual machines.
        :type  depends_on_vm_group: :class:`str` or ``None``
        :param depends_on_vm_group: Depdendency virtual group name. The virtual group may contain one
            or more virtual machines.
        """
        self.status = status
        self.enabled = enabled
        self.in_compliance = in_compliance
        self.name = name
        self.mandatory = mandatory
        self.user_created = user_created
        self.vm_group = vm_group
        self.depends_on_vm_group = depends_on_vm_group
        VapiStruct.__init__(self)


DependencyRule._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.dependency_rule', {
        'status': type.OptionalType(type.ReferenceType(__name__, 'Status')),
        'enabled': type.OptionalType(type.BooleanType()),
        'in_compliance': type.OptionalType(type.BooleanType()),
        'name': type.OptionalType(type.StringType()),
        'mandatory': type.OptionalType(type.BooleanType()),
        'user_created': type.OptionalType(type.BooleanType()),
        'vm_group': type.OptionalType(type.StringType()),
        'depends_on_vm_group': type.OptionalType(type.StringType()),
    },
    DependencyRule,
    False,
    None))



class VmHostRule(VapiStruct):
    """
    The ``VmHostRule`` class defines virtual machines and host groups that
    determine virtual machine placement. The virtual machines and hosts
    referenced by a VM-Host rule must be in the same cluster. 
    
    A VM-Host rule identifies the following groups. 
    
    * A virtual machine group (vim.cluster.VmGroup).
    * Two host groups - an affine host group and an anti-affine host group
      (vim.cluster.HostGroup). At least one of the groups must contain one or
      more hosts.
    
    

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 status=None,
                 enabled=None,
                 in_compliance=None,
                 name=None,
                 mandatory=None,
                 user_created=None,
                 vm_group_name=None,
                 affine_host_group_name=None,
                 anti_affine_host_group_name=None,
                ):
        """
        :type  status: :class:`Status` or ``None``
        :param status: Flag to indicate whether or not the rule is currently satisfied.
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag to indicate whether or not the rule is enabled. Set this
            property when user configures the rule. The default value is false
            (disabled). If there is a rule conflict, the Server can override
            the setting to disable a rule.
            If None or empty, the value is skipped.
        :type  in_compliance: :class:`bool` or ``None``
        :param in_compliance: Flag to indicate if the rule is in compliance.
            If None or empty, the value is skipped.
        :type  name: :class:`str` or ``None``
        :param name: Name of the rule.
        :type  mandatory: :class:`bool` or ``None``
        :param mandatory: Flag to indicate whether compliance with this rule is mandatory or
            optional. The default value is false (optional).
            If None or empty, the value is skipped.
        :type  user_created: :class:`bool` or ``None``
        :param user_created: Flag to indicate whether the rule is created by the user or the
            system.
            If None or empty, the value is skipped.
        :type  vm_group_name: :class:`str` or ``None``
        :param vm_group_name: Virtual group name.
            If None or empty, the value is skipped.
        :type  affine_host_group_name: :class:`str` or ``None``
        :param affine_host_group_name: Name of the affine host group. The affine host group identifies
            hosts on which VmHotRule#vmGroupName virtual machines can be
            powered-on.
            If None or empty, the value is skipped.
        :type  anti_affine_host_group_name: :class:`str` or ``None``
        :param anti_affine_host_group_name: Name of the anti-affine host group. The anti-affine host group
            identifies hosts on which VmHotRule#vmGroupName virtual machines
            should not be powered-on.
            If None or empty, the value is skipped.
        """
        self.status = status
        self.enabled = enabled
        self.in_compliance = in_compliance
        self.name = name
        self.mandatory = mandatory
        self.user_created = user_created
        self.vm_group_name = vm_group_name
        self.affine_host_group_name = affine_host_group_name
        self.anti_affine_host_group_name = anti_affine_host_group_name
        VapiStruct.__init__(self)


VmHostRule._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.vm_host_rule', {
        'status': type.OptionalType(type.ReferenceType(__name__, 'Status')),
        'enabled': type.OptionalType(type.BooleanType()),
        'in_compliance': type.OptionalType(type.BooleanType()),
        'name': type.OptionalType(type.StringType()),
        'mandatory': type.OptionalType(type.BooleanType()),
        'user_created': type.OptionalType(type.BooleanType()),
        'vm_group_name': type.OptionalType(type.StringType()),
        'affine_host_group_name': type.OptionalType(type.StringType()),
        'anti_affine_host_group_name': type.OptionalType(type.StringType()),
    },
    VmHostRule,
    False,
    None))



class Rule(VapiStruct):
    """
    The ``Rule`` class describes affinity and anti-affinity DRS rules that
    affect the placement of virtual machines in a cluster.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 vm_affinity_rules=None,
                 vm_anti_affinity_rules=None,
                 dependency_rule=None,
                 vm_host_rule=None,
                ):
        """
        :type  vm_affinity_rules: :class:`list` of :class:`AffinityRule` or ``None``
        :param vm_affinity_rules: Cluster-wide VM affinity rules.If this is set then
            AntiAffinityRule, :class:`DependencyRule`, :class:`VmHostRule` can
            not be set.
            If None or empty, the value is skipped.
        :type  vm_anti_affinity_rules: :class:`list` of :class:`AffinityRule` or ``None``
        :param vm_anti_affinity_rules: Cluster-wide VM anti affinity rules.If this is set then
            AntiAffinityRule, :class:`DependencyRule`, :class:`VmHostRule` can
            not be set.
            If None or empty, the value is skipped.
        :type  dependency_rule: :class:`list` of :class:`DependencyRule` or ``None``
        :param dependency_rule: Cluster-wide VM-to-VM dependency rules.If this is set then
            :class:`AffinityRule`, AntiAffinityRule, :class:`VmHostRule` can
            not be set.
            If None or empty, the value is skipped.
        :type  vm_host_rule: :class:`list` of :class:`VmHostRule` or ``None``
        :param vm_host_rule: Cluster-wide VM-to-Host affinity rules.If this is set then
            :class:`AffinityRule`, AntiAffinityRule, :class:`DependencyRule`
            can not be set.
            If None or empty, the value is skipped.
        """
        self.vm_affinity_rules = vm_affinity_rules
        self.vm_anti_affinity_rules = vm_anti_affinity_rules
        self.dependency_rule = dependency_rule
        self.vm_host_rule = vm_host_rule
        VapiStruct.__init__(self)


Rule._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.drs.rule', {
        'vm_affinity_rules': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'AffinityRule'))),
        'vm_anti_affinity_rules': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'AffinityRule'))),
        'dependency_rule': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'DependencyRule'))),
        'vm_host_rule': type.OptionalType(type.ListType(type.ReferenceType(__name__, 'VmHostRule'))),
    },
    Rule,
    False,
    None))




class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class DetailedStatus(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`DetailedStatus.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`DetailedStatus.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.detailed_status'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _DetailedStatusStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            tier0_id,
            service_id,
            session_id,
            enforcement_point_path=None,
            source=None,
            ):
        """
        - no enforcement point path specified: detailed tatus is evaluated on
        each enforcement point. - an enforcement point path is specified:
        detailed status is evaluated only on the given enforcement point. -
        source=realtime: detailed tatus is fetched realtime from the
        enforcement point. - source=cached: cached detailed status is returned.

        :type  tier0_id: :class:`str`
        :param tier0_id: (required)
        :type  service_id: :class:`str`
        :param service_id: (required)
        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.AggregateL2VPNSessionStatus`
        :return: com.vmware.nsx_policy.model.AggregateL2VPNSessionStatus
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'tier0_id': tier0_id,
                            'service_id': service_id,
                            'session_id': session_id,
                            'enforcement_point_path': enforcement_point_path,
                            'source': source,
                            })
class PeerConfig(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.peer_config'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _PeerConfigStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            tier0_id,
            service_id,
            session_id,
            enforcement_point_path=None,
            ):
        """
        Get peer config for the Tier-0 L2VPN session to configure the remote
        side of the tunnel. - no enforcement point path specified: L2VPN
        Session Peer Codes will be evaluated on each enforcement point. -
        enforcement point paths specified: L2VPN Session Peer Codes are
        evaluated only on the given enforcement points. API supported only when
        L2VPN Service is in Server Mode.

        :type  tier0_id: :class:`str`
        :param tier0_id: (required)
        :type  service_id: :class:`str`
        :param service_id: (required)
        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.AggregateL2VPNSessionPeerConfig`
        :return: com.vmware.nsx_policy.model.AggregateL2VPNSessionPeerConfig
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'tier0_id': tier0_id,
                            'service_id': service_id,
                            'session_id': session_id,
                            'enforcement_point_path': enforcement_point_path,
                            })
class RemoteMac(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.remote_mac'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _RemoteMacStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            tier0_id,
            service_id,
            session_id,
            enforcement_point_path=None,
            segment_path=None,
            ):
        """
        Returns L2Vpn session remote macs for a logical switch. Data is fetched
        from enforcement point. This API is deprecated. Please use GET
        /infra/tier-0s/<tier-0-id>/ipsec-vpn-services/<service-id>/
        sessions/<session-id>/detailed-status instead.

        :type  tier0_id: :class:`str`
        :param tier0_id: (required)
        :type  service_id: :class:`str`
        :param service_id: (required)
        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  segment_path: :class:`str` or ``None``
        :param segment_path: Segment Path (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.AggregateL2VpnSessionRemoteMac`
        :return: com.vmware.nsx_policy.model.AggregateL2VpnSessionRemoteMac
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'tier0_id': tier0_id,
                            'service_id': service_id,
                            'session_id': session_id,
                            'enforcement_point_path': enforcement_point_path,
                            'segment_path': segment_path,
                            })
class Statistics(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.statistics'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatisticsStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            tier0_id,
            service_id,
            session_id,
            enforcement_point_path=None,
            source=None,
            ):
        """
        - no enforcement point path specified: statistics are evaluated on each
        enforcement point. - an enforcement point path is specified: statistics
        are evaluated only on the given enforcement point. - source=realtime:
        statistics are fetched realtime from the enforcement point. -
        source=cached: cached statistics from the enforcement point are
        returned.

        :type  tier0_id: :class:`str`
        :param tier0_id: (required)
        :type  service_id: :class:`str`
        :param service_id: (required)
        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: String Path of the enforcement point (optional)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.AggregateL2VPNSessionStatistics`
        :return: com.vmware.nsx_policy.model.AggregateL2VPNSessionStatistics
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'tier0_id': tier0_id,
                            'service_id': service_id,
                            'session_id': session_id,
                            'enforcement_point_path': enforcement_point_path,
                            'source': source,
                            })
class _DetailedStatusStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'tier0_id': type.StringType(),
            'service_id': type.StringType(),
            'session_id': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tier-0s/{tier-0-id}/l2vpn-services/{service-id}/sessions/{session-id}/detailed-status',
            path_variables={
                'tier0_id': 'tier-0-id',
                'service_id': 'service-id',
                'session_id': 'session-id',
            },
             header_parameters={
                       },
            query_parameters={
                'enforcement_point_path': 'enforcement_point_path',
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'AggregateL2VPNSessionStatus'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.detailed_status',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _PeerConfigStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'tier0_id': type.StringType(),
            'service_id': type.StringType(),
            'session_id': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tier-0s/{tier-0-id}/l2vpn-services/{service-id}/sessions/{session-id}/peer-config',
            path_variables={
                'tier0_id': 'tier-0-id',
                'service_id': 'service-id',
                'session_id': 'session-id',
            },
             header_parameters={
                     },
            query_parameters={
                'enforcement_point_path': 'enforcement_point_path',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'AggregateL2VPNSessionPeerConfig'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.peer_config',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _RemoteMacStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'tier0_id': type.StringType(),
            'service_id': type.StringType(),
            'session_id': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'segment_path': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tier-0s/{tier-0-id}/l2vpn-services/{service-id}/sessions/{session-id}/remote-mac',
            path_variables={
                'tier0_id': 'tier-0-id',
                'service_id': 'service-id',
                'session_id': 'session-id',
            },
             header_parameters={
                       },
            query_parameters={
                'enforcement_point_path': 'enforcement_point_path',
                'segment_path': 'segment_path',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'AggregateL2VpnSessionRemoteMac'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.remote_mac',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatisticsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'tier0_id': type.StringType(),
            'service_id': type.StringType(),
            'session_id': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/infra/tier-0s/{tier-0-id}/l2vpn-services/{service-id}/sessions/{session-id}/statistics',
            path_variables={
                'tier0_id': 'tier-0-id',
                'service_id': 'service-id',
                'session_id': 'session-id',
            },
             header_parameters={
                       },
            query_parameters={
                'enforcement_point_path': 'enforcement_point_path',
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'AggregateL2VPNSessionStatistics'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.infra.tier_0s.l2vpn_services.sessions.statistics',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'DetailedStatus': DetailedStatus,
        'PeerConfig': PeerConfig,
        'RemoteMac': RemoteMac,
        'Statistics': Statistics,
    }




#!/usr/bin/env python

"""
STDIO server : This handles all protocol requests over stdin/out
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015, 2017 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

import sys

from vmware.vapi.lib.log import get_vapi_logger
from vmware.vapi.server.server_interface import ServerInterface

logger = get_vapi_logger(__name__)


class StdioServer(ServerInterface):
    """ stdio server """

    def __init__(self):
        """
        Initialize the Stdio Server instance
        """
        self.protocol_handler = None
        ServerInterface.__init__(self)

    def register_handler(self, addr, msg_type, protocol_handler, ssl_args=None):
        """
        Returns the struct definition corresponding to the method's input
        parameters. The field names in the struct definition are the parameter
        names and the field values correspond to the data definition of the
        respective fields.

        :type  addr: :class:`str`
        :param addr: Url of the provider
        :type  msg_type: :class:`str`
        :param msg_type: Message Type of the provider
        :type  protocol_handler:
            :class:`vmware.vapi.protocol.server.api_handler.ApiHandler`
        :param protocol_handler: The handler for the message protocol
        :type  ssl_args: :class:`str`
        :param ssl_args: Any ssl related arguments
        """

        self.protocol_handler = protocol_handler

    def serve_forever(self):
        """
        Receives the input from the std in of the process.
        Processes the message, handles it via the protocol_handler and then
        sends the response back.
        """
        msg = sys.stdin.read()
        if msg is None:
            logger.error("No arguments passed to cgi process")
            return
        response = self.protocol_handler.handle_request(msg)
        if response is not None:
            print(response)

    def shutdown(self):
        pass


def get_server():
    """
    get stdio server

    :rtype: :class:`vmware.vapi.server.server_interface.ServerInterface`
    :return: subclass of ServerInterface
    """
    return StdioServer()



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx.vpn.ipsec.sessions.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class State(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.vpn.ipsec.sessions.state'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StateStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            ipsec_vpn_session_id,
            barrier_id=None,
            request_id=None,
            ):
        """
        Return realized state information of a ipsec vpn session. Any
        configuration update that affects the ipsec vpn session can use this
        API to get its realized state by passing a request_id returned by the
        configuration change operation. e.g. Update configuration of ipsec vpn
        session, service, endpoints, profiles, etc. It will return a service
        disabled error, if the ipsec vpn service associated with the session is
        disabled. 
        
        Please use below Policy APIs. 
        GET
        /policy/api/v1/infra/realized-state/realized-entity?realized_path=/policy/api/v1/infra/tier-0s/<tier-0-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>
        GET
        /policy/api/v1/infra/realized-state/realized-entity?realized_path=/policy/api/v1/infra/tier-1s/<tier-1-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>

        .. deprecated:: unknown

        :type  ipsec_vpn_session_id: :class:`str`
        :param ipsec_vpn_session_id: (required)
        :type  barrier_id: :class:`long` or ``None``
        :param barrier_id: (optional)
        :type  request_id: :class:`str` or ``None``
        :param request_id: Realization request ID (optional)
        :rtype: :class:`com.vmware.nsx.model_client.IPSecVPNSessionState`
        :return: com.vmware.nsx.model.IPSecVPNSessionState
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.vpn.ipsec.sessions.State.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'ipsec_vpn_session_id': ipsec_vpn_session_id,
                            'barrier_id': barrier_id,
                            'request_id': request_id,
                            })
class Statistics(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.vpn.ipsec.sessions.statistics'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatisticsStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            session_id,
            source=None,
            ):
        """
        Get statistics of a vpn session across all tunnels and IKE session.
        Query parameter \"source=realtime\" is the only supported source. 
        
        Please use below Policy APIs. 
        GET
        /policy/api/v1/infra/tier-0s/<tier-0-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/statistics
        GET
        /policy/api/v1/infra/tier-1s/<tier-1-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/statistics

        .. deprecated:: unknown

        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.IPSecVPNSessionStatistics`
        :return: com.vmware.nsx.model.IPSecVPNSessionStatistics
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.vpn.ipsec.sessions.Statistics.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'session_id': session_id,
                            'source': source,
                            })

    def reset(self,
              session_id,
              ):
        """
        Reset the statistics of the given VPN session. 
        
        Please use below Policy APIs. 
        POST
        /policy/api/v1/infra/tier-0s/<tier-0-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/statistics
        POST
        /policy/api/v1/infra/tier-1s/<tier-1-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/statistics

        .. deprecated:: unknown

        :type  session_id: :class:`str`
        :param session_id: (required)
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.vpn.ipsec.sessions.Statistics.reset is deprecated.', DeprecationWarning)
        return self._invoke('reset',
                            {
                            'session_id': session_id,
                            })
class Status(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.vpn.ipsec.sessions.status'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatusStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            session_id,
            source=None,
            ):
        """
        List status of IPSec session. Query parameter source supports both
        realtime and cached mode. 
        
        Please use below Policy APIs. 
        GET
        /policy/api/v1/infra/tier-0s/<tier-0-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/detailed-status
        GET
        /policy/api/v1/infra/tier-1s/<tier-1-id>/ipsec-vpn-services/<service-id>/sessions/<session-id>/detailed-status

        .. deprecated:: unknown

        :type  session_id: :class:`str`
        :param session_id: (required)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.IPSecVPNSessionStatus`
        :return: com.vmware.nsx.model.IPSecVPNSessionStatus
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.vpn.ipsec.sessions.Status.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'session_id': session_id,
                            'source': source,
                            })
class Summary(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Summary.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Summary.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.vpn.ipsec.sessions.summary'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _SummaryStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            site_id=None,
            source=None,
            ):
        """
        VPN session summary gets summary per vpn sessions and IKE session.
        Query parameter source supports only cached mode. 
        
        The information is already displayed on UI.

        .. deprecated:: unknown

        :type  site_id: :class:`str` or ``None``
        :param site_id: Peer site id (optional)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.IPSecVPNSessionSummary`
        :return: com.vmware.nsx.model.IPSecVPNSessionSummary
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.vpn.ipsec.sessions.Summary.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'site_id': site_id,
                            'source': source,
                            })
class _StateStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'ipsec_vpn_session_id': type.StringType(),
            'barrier_id': type.OptionalType(type.IntegerType()),
            'request_id': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/vpn/ipsec/sessions/{ipsec-vpn-session-id}/state',
            path_variables={
                'ipsec_vpn_session_id': 'ipsec-vpn-session-id',
            },
             header_parameters={
                   },
            query_parameters={
                'barrier_id': 'barrier_id',
                'request_id': 'request_id',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'IPSecVPNSessionState'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.vpn.ipsec.sessions.state',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatisticsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'session_id': type.StringType(),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/vpn/ipsec/sessions/{session-id}/statistics',
            path_variables={
                'session_id': 'session-id',
            },
             header_parameters={
                 },
            query_parameters={
                'source': 'source',
            },
            content_type='application/json'
        )

        # properties for reset operation
        reset_input_type = type.StructType('operation-input', {
            'session_id': type.StringType(),
        })
        reset_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        reset_input_value_validator_list = [
        ]
        reset_output_validator_list = [
        ]
        reset_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/api/v1/vpn/ipsec/sessions/{session-id}/statistics?action=reset',
            path_variables={
                'session_id': 'session-id',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'IPSecVPNSessionStatistics'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'reset': {
                'input_type': reset_input_type,
                'output_type': type.VoidType(),
                'errors': reset_error_dict,
                'input_value_validator_list': reset_input_value_validator_list,
                'output_validator_list': reset_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'reset': reset_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.vpn.ipsec.sessions.statistics',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatusStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'session_id': type.StringType(),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/vpn/ipsec/sessions/{session-id}/status',
            path_variables={
                'session_id': 'session-id',
            },
             header_parameters={
                 },
            query_parameters={
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'IPSecVPNSessionStatus'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.vpn.ipsec.sessions.status',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _SummaryStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'site_id': type.OptionalType(type.StringType()),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/vpn/ipsec/sessions/summary',
            path_variables={
            },
             header_parameters={
                 },
            query_parameters={
                'site_id': 'site_id',
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'IPSecVPNSessionSummary'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.vpn.ipsec.sessions.summary',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'State': State,
        'Statistics': Statistics,
        'Status': Status,
        'Summary': Summary,
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx_vmc_app.infra.direct_connect.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Bgp(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_vmc_app.infra.direct_connect.bgp'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _BgpStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self):
        """
        Get Direct Connect BGP related information, including current
        Autonomous System Number of the VGW attached to the VPC


        :rtype: :class:`com.vmware.nsx_vmc_app.model_client.DirectConnectBgpInfo`
        :return: com.vmware.nsx_vmc_app.model.DirectConnectBgpInfo
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get', None)

    def patch(self,
              direct_connect_bgp_info,
              ):
        """
        This API is used to update DX BGP related information. For ASN update,
        VIFs should be disconnected from the DX VGW before making this API
        call. The ASN update operation will be synchronous at this point. In
        the future the user should make use of the Get RealizationStatus call
        to check update status. While an ASN update call is in progress, any
        other DX BGP update request will be rejected.

        :type  direct_connect_bgp_info: :class:`com.vmware.nsx_vmc_app.model_client.DirectConnectBgpInfo`
        :param direct_connect_bgp_info: (required)
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('patch',
                            {
                            'direct_connect_bgp_info': direct_connect_bgp_info,
                            })

    def update(self,
               direct_connect_bgp_info,
               ):
        """
        This API is used to create DX BGP related information.

        :type  direct_connect_bgp_info: :class:`com.vmware.nsx_vmc_app.model_client.DirectConnectBgpInfo`
        :param direct_connect_bgp_info: (required)
        :rtype: :class:`com.vmware.nsx_vmc_app.model_client.DirectConnectBgpInfo`
        :return: com.vmware.nsx_vmc_app.model.DirectConnectBgpInfo
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('update',
                            {
                            'direct_connect_bgp_info': direct_connect_bgp_info,
                            })
class Vifs(VapiInterface):
    """
    
    """
    CREATE_ACTION_ATTACH = "ATTACH"
    """
    Possible value for ``action`` of method :func:`Vifs.create`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_vmc_app.infra.direct_connect.vifs'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _VifsStub)
        self._VAPI_OPERATION_IDS = {}


    def create(self,
               vif_id,
               action,
               ):
        """
        Attach the specified VIF (Virtual Interface) to the SDDC.

        :type  vif_id: :class:`str`
        :param vif_id: (required)
        :type  action: :class:`str`
        :param action: Direct Connect VIF (Virtual Interface) action (required)
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('create',
                            {
                            'vif_id': vif_id,
                            'action': action,
                            })

    def delete(self,
               vif_id,
               ):
        """
        Detach and delete a Direct Connect VIF (Virtual Interface) from the
        SDDC. Detach and delete are one operation in VMC provider, a Direct
        Connect VIF will be deleted after detached.

        :type  vif_id: :class:`str`
        :param vif_id: (required)
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('delete',
                            {
                            'vif_id': vif_id,
                            })

    def list(self):
        """
        Return all non-connected VIFs (with states \"avalible\", \"down\",
        \"pending\" and \"confirming\") and connected VIFs that are available
        to the SDDC.


        :rtype: :class:`com.vmware.nsx_vmc_app.model_client.VifsListResult`
        :return: com.vmware.nsx_vmc_app.model.VifsListResult
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('list', None)
class _BgpStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {})
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/cloud-service/api/v1/infra/direct-connect/bgp',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for patch operation
        patch_input_type = type.StructType('operation-input', {
            'direct_connect_bgp_info': type.ReferenceType('com.vmware.nsx_vmc_app.model_client', 'DirectConnectBgpInfo'),
        })
        patch_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        patch_input_value_validator_list = [
        ]
        patch_output_validator_list = [
        ]
        patch_rest_metadata = OperationRestMetadata(
            http_method='PATCH',
            url_template='/cloud-service/api/v1/infra/direct-connect/bgp',
            request_body_parameter='direct_connect_bgp_info',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for update operation
        update_input_type = type.StructType('operation-input', {
            'direct_connect_bgp_info': type.ReferenceType('com.vmware.nsx_vmc_app.model_client', 'DirectConnectBgpInfo'),
        })
        update_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        update_input_value_validator_list = [
        ]
        update_output_validator_list = [
        ]
        update_rest_metadata = OperationRestMetadata(
            http_method='PUT',
            url_template='/cloud-service/api/v1/infra/direct-connect/bgp',
            request_body_parameter='direct_connect_bgp_info',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_vmc_app.model_client', 'DirectConnectBgpInfo'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'patch': {
                'input_type': patch_input_type,
                'output_type': type.VoidType(),
                'errors': patch_error_dict,
                'input_value_validator_list': patch_input_value_validator_list,
                'output_validator_list': patch_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'update': {
                'input_type': update_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_vmc_app.model_client', 'DirectConnectBgpInfo'),
                'errors': update_error_dict,
                'input_value_validator_list': update_input_value_validator_list,
                'output_validator_list': update_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'patch': patch_rest_metadata,
            'update': update_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_vmc_app.infra.direct_connect.bgp',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _VifsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'vif_id': type.StringType(),
            'action': type.StringType(),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/cloud-service/api/v1/infra/direct-connect/vifs/{vif-id}',
            path_variables={
                'vif_id': 'vif-id',
            },
             header_parameters={
                 },
            query_parameters={
                'action': 'action',
            },
            content_type='application/json'
        )

        # properties for delete operation
        delete_input_type = type.StructType('operation-input', {
            'vif_id': type.StringType(),
        })
        delete_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        delete_input_value_validator_list = [
        ]
        delete_output_validator_list = [
        ]
        delete_rest_metadata = OperationRestMetadata(
            http_method='DELETE',
            url_template='/cloud-service/api/v1/infra/direct-connect/vifs/{vif-id}',
            path_variables={
                'vif_id': 'vif-id',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/cloud-service/api/v1/infra/direct-connect/vifs',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'create': {
                'input_type': create_input_type,
                'output_type': type.VoidType(),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': create_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'delete': {
                'input_type': delete_input_type,
                'output_type': type.VoidType(),
                'errors': delete_error_dict,
                'input_value_validator_list': delete_input_value_validator_list,
                'output_validator_list': delete_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'list': {
                'input_type': list_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_vmc_app.model_client', 'VifsListResult'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'create': create_rest_metadata,
            'delete': delete_rest_metadata,
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_vmc_app.infra.direct_connect.vifs',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Bgp': Bgp,
        'Vifs': Vifs,
        'routes': 'com.vmware.nsx_vmc_app.infra.direct_connect.routes_client.StubFactory',
    }

